using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using VaTS;
using Ato.EN.IntegrationServices.CodeGenerationRS;
using DataContracts;
using Ato.EN.IntegrationServices.CodeGenerationTRT;

namespace Ato.CD.Inbound.TRT201902
{
    internal class CrossFormValidatorRS : ICrossFormValidator
    {
        private TRT2019 ParentDocument { get; }

        private RS2018 ChildDocument { get; }

        internal CrossFormValidatorRS(TRT2019 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (RS2018)childDocument.ConsumedReport;
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOGEN402009(response); 
            VRATORS000150(response);
            VRATORS000137(response);
            VRATORS000138(response);

            return response;
        } 

        #region VR.ATO.GEN.402009

        /*  VR.ATO.GEN.402009
                    The Entity Identifier on the Schedule must equal the Entity Identifier on the Parent Return

                Legacy Rule Format:
                    IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                    RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^TFN <> ^ParentTFN

        Data Elements:
            
                    ^ParentTFN = ParentTFN
            
                    ^TFN = RP
                    */
        private void VRATOGEN402009(List<ProcessMessageDocument> response )
        {
            bool assertion = !ParentDocument.RPIdentifierTFN.Equals(ChildDocument.RPIdentifierTFN);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    LongDescription = @"Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.402009

        #region VR.ATO.RS.000150

        /*  VR.ATO.RS.000150
                    The Entity name must be the same as the name on the main form

                Legacy Rule Format:
                    IF (PARENT RETURN <> SET("IITR","TRTAMI") AND [RS14] <> PARENT RETURN(pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text (IN TUPLE(xbrli\organisationame2.xx.xx:OrganisationNameDetails) WHERE (pyde.xx.xx:OrganisationNameDetails.OrganisationalType.Code = "MN") AND (pyde.xx.xx:OrganisationNameDetails.Currency.Code = "C")))) OR (PARENT RETURN = SET("IITR","TRTAMI") AND [RS14] <> PARENT RETURN:RP:pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RS14] = RS:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text

            Technical Business Rule Format:
                    (PARENT <> SET("IITR","TRTAMI") AND ^RS14 <> ^ParentORGN.T) OR (PARENT = SET("IITR","TRTAMI") AND ^RS14 <> PARENT:^RS14)

        Data Elements:
            
                    RP:^RS14 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text
            
                    RP:^ParentORGN.T = OrganisationNameDetails.OrganisationalName.Text
                    */
        private void VRATORS000150(List<ProcessMessageDocument> response)
        {
            bool assertion = !ParentDocument.TRT6.Equals(ChildDocument.RS14);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.RS.000150",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The Entity name must be the same as the name on the main form",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.RS.000150" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.RS.000150

        #region VR.ATO.RS.000137

        /*  VR.ATO.RS.000137
                    Number of weeks property rented must be equal to or less than 52

                Legacy Rule Format:
                    IF (PARENT RETURN <> "IITR") AND ([RS25] > 52)  
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RS25] = RS:RP:bafot.02.00:RealEstateProperty.RentalWeeks.Count

            Technical Business Rule Format:
                    (PARENT <> 'IITR') AND (^RS25 > 52)

        Data Elements:
            
                    RP:^RS25 = bafot.02.00:RealEstateProperty.RentalWeeks.Count
                    */
        private void VRATORS000137(List<ProcessMessageDocument> response )
        {
            bool assertion = ChildDocument.RS25.GetValueOrDefault() > 52 ;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.RS.000137",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of weeks property rented must not exceed 52",
                    Location = "/xbrli:xbrl/tns:RealEstateProperty.RentalWeeks.Count",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.RS.000137" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.RS.000137

        #region VR.ATO.RS.000138

        /*  VR.ATO.RS.000138
                    Number of weeks property was available for rent this year must be equal to or less than 52

                Legacy Rule Format:
                    IF (PARENT RETURN <> "IITR") AND ([RS27] > 52)  
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RS27] = RS:RP:bafot.02.00:RealEstateProperty.RentalWeeksAvailable.Count

            Technical Business Rule Format:
                    (PARENT <> 'IITR') AND (^RS27 > 52)

        Data Elements:
            
                    RP:^RS27 = bafot.02.00:RealEstateProperty.RentalWeeksAvailable.Count
                    */
        private void VRATORS000138(List<ProcessMessageDocument> response )
        {
            bool assertion = ChildDocument.RS27.GetValueOrDefault() > 52;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.RS.000138",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of weeks property was available for rent this year cannot exceed 52 weeks",
                    Location = "/xbrli:xbrl/tns:RealEstateProperty.RentalWeeksAvailable.Count",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.RS.000138" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.RS.000138 
    }
}